function isInApp(){
	return typeof AppsgeyserJSInterface != 'undefined';
}

var currentColor = [255,0,0];
var currentPicture = null;
var paletteScroll;
$(function(){
	generatePalette();
	document.addEventListener('touchmove', function (e) {
		if(!$('.gallery-page').visible()){
			e.preventDefault();
		}
	}, false);

	if(isInApp() && !AppsgeyserJSInterface.saveImageFromBase64){
		$('.save').hide();
	}
	generateToolStyles();
	function onResize(){
		var outerHeight = $('.tools-palette').outerHeight();
		if(outerHeight == 0){
			setTimeout(onResize,50);
			return;
		}
		var size =  outerHeight - 20;
		$('.color, .remove, .gallery, .open-palette, .save').css({'height':size, width:size});
	}
	onResize();
	$(window).bind('resize',onResize);
	initGallery();
	loadPicture(images[0],initWithPicture);
	var $loading = $('.loading');
	$('#coloring-canvas').bind('click',function(e){
		$loading.show();
		var local = getLocalCoords(this, getEventCoords(e));
		setTimeout(function(){
			floodFill(local.x|0,local.y|0);
			$loading.hide();
		},50);
	});

	$('.color').click(function(){
		$('.color').removeClass('selected');
		var $elem = $(this);
		$elem.addClass('selected');
		currentColor = colorFromElem($elem);
	});

	$('.save').click(function(){
		if(isInApp()){
			AppsgeyserJSInterface.saveImageFromBase64($('#coloring-canvas')[0].toDataURL(),"Coloring");
		} else {
			window.location = $('#coloring-canvas')[0].toDataURL();
		}
	});

	$('.gallery').click(function(){
		$('.main-page').hide();
		$('.gallery-page').show();
	});
	$('.gallery-page').on('click','.gallery-image',function(){
		loadPicture($(this).attr('src'),initWithPicture);
		$('.main-page').show();
		$('.gallery-page').hide();
	});

	$('.remove').click(function(){
		if(confirm("This will erase current picture colors. Are you sure?")){
			initWithPicture(currentPicture);
		}
	});

	$('.open-palette').click(function(){
		$('.palette').show();
		if(!paletteScroll){
			paletteScroll = new iScroll('palette');
		}
	});
	$('.palette').on('click','.color',function(){
		$('.palette').hide();
		$('.tools .color:last').remove();
		$('.tools .color:first').before(this);
	});

});

function generatePalette(){
	var $palette = $('#scroller');
	var colors = [];
	for(var x = 45; x <= 255; x+=35){
		for(var y = 45; y <=255; y+=35){
			for(var z=45; z <=255; z+=35){
				colors.push([x,y,z]);
			}
		}
	}
	colors.sort(function(a,b){
		return (b[0]+b[1]+b[2]) - (a[0]+a[1]+a[2]);
	});
	for(var i in colors){
		if(!colors.hasOwnProperty(i)) continue;
		$('<div>')
			.addClass("color")
			.attr('data-color', colors[i][0]+','+colors[i][1]+','+colors[i][2])
			.appendTo($palette);
	}
}

function initGallery(){
	for(var i in images){
		if(!images.hasOwnProperty(i)) continue;
		$('<img>')
			.attr('src',images[i])
			.addClass('gallery-image')
			.appendTo('.gallery-page');
	}
}

function colorFromElem($elem){
	var colorText = $elem.attr('data-color');
	return colorText.split(',').map(function(a){return parseInt(a)});
}

function generateToolStyles(){
	$('.color').each(function(index,elem){
		var $elem = $(elem);
		var color = colorFromElem($elem);
		var gradient = generateGradient(color);
		$elem.attr('style',gradient);
	});
}

function generateGradient(color){
	var cssTemplate = 'background: %from%;\
	background: -webkit-gradient(linear, left top, left bottom, color-stop(0%,%from%), color-stop(100%, %to%));\
	background: -webkit-linear-gradient(top,  %from% 0%,%to% 100%);\
	background: linear-gradient(to bottom,  %from% 0%,%to% 100%);\
	border-color: %to%';

	var from = 'rgb(' + color[0] + ',' + color[1] + ',' + color[2] + ')';
	color[0]= color[0]*0.3|0;
	color[1]=color[1]*0.3|0;
	color[2]=color[2]*0.3|0;
	var to = 'rgb(' + color[0] + ',' + color[1] + ',' + color[2] + ')';
	return cssTemplate.replace(/%from%/g,from).replace(/%to%/g,to);
}


Array.prototype.shuffle = function (b) {
	var i = this.length, j, t;
	while (i) {
		j = Math.floor(( i-- ) * Math.random());
		t = b && typeof this[i].shuffle !== 'undefined' ? this[i].shuffle() : this[i];
		this[i] = this[j];
		this[j] = t;
	}

	return this;
};

function loadPicture(imagePath,callback) {
	$.when($.loadImage(imagePath)).done(function (imageLoaded) {
		/*var screenWidth = document.body.offsetWidth * window.devicePixelRatio;
		if (imageLoaded.width > screenWidth) {
			var factor = screenWidth / imageLoaded.width;
			imageLoaded.width *= factor;
			imageLoaded.height *= factor;
		}*/
		if (imageLoaded.width > 800) {
			var factor = 800 / imageLoaded.width;
			imageLoaded.width *= factor;
			imageLoaded.height *= factor;
		}
		if(typeof callback == 'function') callback(imageLoaded);
	});
}
//var imageZoom = null;
function initWithPicture(picture){
	drawOnCanvas(picture);
	currentPicture = picture;
//	if(!imageZoom) imageZoom = new iScroll('canvas-wrapper', { zoom:true, hScrollbar:false, vScrollbar:false });
}

function drawOnCanvas(imageLoaded){
	var canvas = $('#coloring-canvas')[0];
	var ctx = canvas.getContext('2d');
	canvas.width = imageLoaded.width;
	canvas.height = imageLoaded.height;
	ctx.drawImage(imageLoaded,0,0,imageLoaded.width,imageLoaded.height);
}

function getEventCoords(ev) {
	var first, coords = {};
	var origEv = ev.originalEvent; // get from jQuery

	if (origEv.changedTouches != undefined) {
		first = origEv.changedTouches[0];
		coords.pageX = first.pageX;
		coords.pageY = first.pageY;
	} else {
		coords.pageX = ev.pageX;
		coords.pageY = ev.pageY;
	}

	return coords;
}

function getLocalCoords(elem, coords) {
/*	var ox = 0, oy = 0;*/
	var offset = $(elem).offset();
/*	while (elem != null) {
		ox += elem.offsetLeft *//*+ parseInt($(elem).css("padding-left"))*//*;
		oy += elem.offsetTop *//*+ parseInt($(elem).css("padding-top"))*//*;
		elem = elem.offsetParent;
	}*/
	var x = coords.pageX - offset.left;
	var y = coords.pageY - offset.top;
	var $canvas = $('#coloring-canvas');
	var factor = $canvas[0].width / (parseInt($canvas.width()));
	x = - parseInt($canvas.css('padding-left')) + x*factor;
	y = - parseInt($canvas.css('padding-top')) + y*factor;
	return { 'x': x, 'y': y};
}

function floodFill(x,y){
	floodFillCanvas($('#coloring-canvas')[0],x,y,currentColor);
}

function floodFillCanvas(canvas, x, y, replacementColor) {
	var current, w, e, stack, cx, cy;
	var context = canvas.getContext("2d");
	var pixelData = context.getImageData(0, 0, canvas.width, canvas.height);
	var width = canvas.width;
	var p = pixelData.data;

	function getPixelIndex(x,y){
		return (y * width + x) * 4;
	}

	function isPixelWhite(i){
		return p[i]>= 30 || p[i+1] >= 30 || p[i+2] >= 30;
	}

	function isPixelBlack(i){
		return p[i] < 30 && p[i+1] < 30 && p[i+2] < 30;
	}

	function paintPixel(i, color){
		p[i] = color[0];
		p[i+1] = color[1];
		p[i+2] = color[2];
	}

	function isPixel(i, color){
		return (p[i] == color[0] && p[i+1] == color[1] && p[i+2] == color[2]);
	}

	var done = [];

	var targetPixel = getPixelIndex(x,y,pixelData);
	if(isPixelBlack(targetPixel)) return;

	stack = [ [x, y] ];
	done[x + y*width] = true;
	while ((current = stack.pop())) {
		cx = current[0];
		cy = current[1];

		var pixelIndex = getPixelIndex(cx,cy);
		if (isPixelWhite(pixelIndex) && !isPixel(pixelIndex,replacementColor)) {
			paintPixel(pixelIndex,replacementColor);

			w = e = cx;
			while (w > 0 && isPixelWhite(getPixelIndex(w-1,cy))) {
				--w;
				if (done[w + cy*width]) break;
				paintPixel(getPixelIndex(w,cy),replacementColor);
			}
			while (e < width - 1 && isPixelWhite(getPixelIndex(e+1,cy))) {
				++e;
				if (done[e + cy* width]) break;
				paintPixel(getPixelIndex(e,cy),replacementColor);
			}

			for (cx = w; cx <= e; cx++) {
				if (cy > 0) {
					pixelIndex = getPixelIndex(cx,cy-1);
					if (isPixelWhite(pixelIndex)) {
						if (!done[cx + (cy - 1)*width]) {
							stack.push([cx, cy - 1]);
							done[cx + (cy - 1)*width] = true;
						}
					}
				}
				if (cy < canvas.height - 1) {
					pixelIndex = getPixelIndex(cx,cy+1);
					if (isPixelWhite(pixelIndex)) {
						if (!done[cx + (cy + 1)*width]) {
							stack.push([cx, cy + 1]);
							done[cx + (cy + 1)*width] = true;
						}
					}
				}
			}
		}
	}

	context.putImageData(pixelData, 0, 0, 0, 0, canvas.width, canvas.height);
}